
type ui_main_function = ?level:int -> Deck.t -> unit;;

let registry = Hashtbl.create 5

let register_ui name main =
  Hashtbl.add registry name main

let get_ui name =
  Hashtbl.find registry name

let ui_list () =
  Hashtbl.fold 
    (fun key value li -> key::li)
    registry
    []
